#include "graphic.h"

void
Timeout_callback(int id, void * fdui){
  FD_MAIN * Main = (FD_MAIN *) fdui;
  
  Draw_new_position();
  Draw_memory_position(current_position);

  Main->ldata = fl_add_timeout(DELAY, Timeout_callback, Main);
  return;
}

void
Start_callback(FL_OBJECT *Start, long NONE) {
  printf("Button Start\n");

  if(FLAG_LOADED) Explorer();
}

void
Reset_callback(FL_OBJECT *Reset, long NONE) {
  
  int ligne, colonne;
  
  FL_OBJECT *obj;
  
  fl_freeze_form(fd_Main->main);

  for(ligne = 0; ligne < NB_LIG; ligne++)
    for(colonne = 0; colonne < NB_COL; colonne++){
      obj = fd_Main->Mem[colonne+(ligne*NB_LIG)];
      fl_set_pixmap_file(obj, NULL_PIXMAP);
    }
  
  for(ligne = 0; ligne < NB_LIG; ligne++)
    for(colonne = 0; colonne < NB_COL; colonne++){
      obj = fd_Main->Carre[colonne+(ligne*NB_LIG)];
      switch(map[colonne+(ligne*NB_LIG)])
	{
	case '0': fl_set_pixmap_file(obj, FLOOR_PIXMAP); break;
	case '1': break; // WALL dj fait 
	case '2': fl_set_pixmap_file(obj, DROID_PIXMAP); break;
	case '3': fl_set_pixmap_file(obj, EXIT_PIXMAP); break;
	case '4': fl_set_pixmap_file(obj, TREASURE_PIXMAP); break; 
	}
    }
  
  last_position = current_position = 1;
  
  fl_unfreeze_form(fd_Main->main);
}

void
Quit_callback(FL_OBJECT *Quit, long NONE) {
  FD_QUIT *fd_Quit;
  
  printf("Button Quit\n");
  fd_Quit = create_form_quit();
  fl_show_form(fd_Quit->quit, FL_PLACE_CENTER, FL_FULLBORDER, "Quit Project");
}

void
Quit_Yes_callback(FL_OBJECT *Quit, long NONE) {
  printf("Button Quit_Yes\n");
  
  exit(0); // avant de trouver mieux
}

void
Quit_No_callback(FL_OBJECT *Quit, long NONE) {  
  FD_QUIT *fd_Quit = Quit->form->fdui;
  
  printf("Button Quit_No\n");
  
  fl_hide_form(fd_Quit->quit);
  fl_free_form(fd_Quit->quit);
}

void
Game_Menu_callback(FL_OBJECT * Game_Menu, long NONE) {
  int Item = fl_get_menu(Game_Menu);
  FD_BROWSER *fd_Browser;
  FD_QUIT *fd_Quit;
  
  switch(Item) 
    {
    default :
      return;
    case 1 :
      printf("Start Game\n");
      if(FLAG_LOADED) Explorer();
      break;
    case 2:
      printf("Load Map\n");
      fd_Browser = create_form_browser();
      fl_show_form(fd_Browser->browser, FL_PLACE_CENTER, FL_FULLBORDER, "Browser Project");
      break;
    case 3:
      printf("Quit\n");
      fd_Quit = create_form_quit();
      fl_show_form(fd_Quit->quit, FL_PLACE_CENTER, FL_FULLBORDER, "Quit Project");
      break;
    }
}

void
Options_Menu_callback(FL_OBJECT * Options_Menu, long NONE) {
  int Item = fl_get_menu(Options_Menu);

  pthread_t th; // thread pour le serveur

  switch(Item) 
    {
    default :
      return;
    case 1 :
      printf("Labyrinth\n");
      break;
    case 2 :
      printf("Robot\n");
      break;
    case 3 :  
      // lancement du serveur
      if (pthread_create (&th, NULL, network, (void *) 0) < 0) {
	fprintf (stderr, "Erreur de creation du thread\n");
	exit (1);
      }
      printf("Network Started\n");
      pthread_mutex_init(&screenlock, NULL);
      fd_Main->ldata = fl_add_timeout(DELAY, Timeout_callback, fd_Main);      
      break;
    }
}

void
About_Menu_callback(FL_OBJECT * About_Menu, long NONE) {
  int Item = fl_get_menu(About_Menu);
  FD_ABOUT *fd_About;
  
  switch(Item) 
    {
    default :
      return;
    case 1 :
      printf("Info\n");
      fd_About = create_form_about();
      break;
    }
}

void
Load_Browser_callback(FL_OBJECT *ob, long NONE) {
  const char *map_file = NULL;
  FD_BROWSER *fdui = ob->form->fdui;
  
  printf("Button Load_Browser\n");
  
  map_file = fl_get_input(fdui->Input_map);
  Load_Map(map_file);
  Draw_Map();

  fl_hide_form(fdui->browser);
  fl_free_form(fdui->browser);
}

void
Browse_Browser_callback(FL_OBJECT *ob, long NONE) {
  const char *fname = NULL;
  FD_BROWSER *fdui = ob->form->fdui;
  
  printf("Button Browse_Browser\n");

  fl_set_fselector_title("Map Selector Project");
  fl_set_fselector_placement(FL_PLACE_CENTER);
  fl_set_fselector_fontsize(FL_NORMAL_SIZE);  // redimensionne le texte
  fl_set_fselector_fontstyle(FL_BOLD_STYLE);  // texte en gras

  if((fname = fl_show_fselector("Map To Load","","*.lab","")))
    fl_set_input(fdui->Input_map, fname);
}

void
Exit_Browser_callback(FL_OBJECT *ob, long NONE) {
  FD_BROWSER *fd_Browser = ob->form->fdui;

  printf("Button Exit_Browser\n");

  fl_hide_form(fd_Browser->browser);
  fl_free_form(fd_Browser->browser);
}

void
About_Ok_callback(FL_OBJECT *ob, long NONE) {
  FD_ABOUT *fd_About = ob->form->fdui;

  printf("Button Ok_About\n");

  fl_hide_form(fd_About->about);
  fl_free_form(fd_About->about);
}
 
// fonction pour joystick
void
move_top_cb(FL_OBJECT *ob, long NONE){

  FL_OBJECT *obj;

  fl_freeze_form(fd_Main->main);

  if(map[current_position - NB_COL] != '1')
    {
      // enlve ancien droid
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, FLOOR_PIXMAP);
      
      // pose le nouveau
      current_position -= NB_COL;
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, DROID_PIXMAP);
    }
  fl_unfreeze_form(fd_Main->main);

  Draw_memory_position(current_position);
}

void
move_left_cb(FL_OBJECT *ob, long NONE){
  FL_OBJECT *obj;

  fl_freeze_form(fd_Main->main);

  if(map[current_position-1] != '1')
    {
      // enlve ancien droid
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, FLOOR_PIXMAP);
      
      // pose le nouveau
      current_position -= 1;
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, DROID_PIXMAP);
    }  
  fl_unfreeze_form(fd_Main->main);  

  Draw_memory_position(current_position);
}

void
move_right_cb(FL_OBJECT *ob, long NONE){

  FL_OBJECT *obj;

  fl_freeze_form(fd_Main->main);

  if(map[current_position+1] != '1')
    {
      // enlve ancien droid
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, FLOOR_PIXMAP);
      
      // pose le nouveau
      current_position += 1;
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, DROID_PIXMAP);
    }    
  fl_unfreeze_form(fd_Main->main);

  Draw_memory_position(current_position);
}

void
move_bottom_cb(FL_OBJECT *ob, long NONE){
  
  FL_OBJECT *obj;

  fl_freeze_form(fd_Main->main);

  if(map[current_position + NB_COL] != '1')
    {
      // enlve ancien droid
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, FLOOR_PIXMAP);
      
      // pose le nouveau
      current_position += NB_COL;
      obj = fd_Main->Carre[current_position];
      fl_set_pixmap_file(obj, DROID_PIXMAP);
    }      
  fl_unfreeze_form(fd_Main->main);

  Draw_memory_position(current_position);
}



