#include <forms.h>
#include <stdlib.h>
#include <pthread.h>

// dimensions de la map et de la grille
#define MAX_INIT 5
#define NB_COL 20
#define NB_LIG 20

#define GRID_DX 20
#define GRID_DY 20
#define MEM_DX 12
#define MEM_DY 12

#define FLOOR 0
#define WALL 1
#define INPUT 2
#define EXIT 3
#define TREASURE 4

// images ncessaires environnement
#define LOAD_PIXMAP   "images/Load_Pixmap.xpm"
#define ICON_PIXMAP   "images/Icon.xpm"
#define START_BUTTON  "images/Start_Button.xpm"
#define RESET_BUTTON  "images/Reset_Button.xpm"
#define QUIT_BUTTON   "images/Quit_Button.xpm"
#define LOAD_BUTTON   "images/Load_Button.xpm"
#define BROWSE_BUTTON "images/Browse_Button.xpm"
#define EXIT_BUTTON   "images/Exit_Button.xpm"
#define YES_BUTTON    "images/Yes_Button.xpm"
#define NO_BUTTON     "images/No_Button.xpm"
#define OK_BUTTON     "images/Ok_Button.xpm"

// images ncessaires map
#define FLOOR_PIXMAP    "images/Floor_Pixmap.xpm"
#define WALL_PIXMAP     "images/Wall_Pixmap.xpm"
#define INPUT_PIXMAP    "images/Input_Pixmap.xpm"
#define EXIT_PIXMAP     "images/Exit_Pixmap.xpm"
#define TREASURE_PIXMAP "images/Treasure_Pixmap.xpm"
#define DROID_PIXMAP    "images/Small.xpm"

// images ncessaires map_memory
#define INODE_PIXMAP   "images/Inode_Pixmap.xpm"
#define INODE_TREASURE "images/Inode_Treasure.xpm"
#define INODE_EXIT     "images/Inode_Exit.xpm"
#define NULL_PIXMAP    "images/Null_Pixmap.xpm"

// definitions des directions
#define TOWARD_LEFT  1
#define TOWARD_DOWN  2
#define TOWARD_RIGHT 3
#define TOWARD_UP    4

// pour le joystick
#define MOVE_TOP    104
#define MOVE_LEFT   101 
#define MOVE_RIGHT  103
#define MOVE_BOTTOM 102

//definitions pour les capteurs
#define LEFT             10
#define LEFT_RIGHT       11
#define LEFT_FRONT       12
#define LEFT_RIGHT_FRONT 13
#define FRONT            15
#define RIGHT            20
#define RIGHT_FRONT      21
#define T                0

//interval de rafraichissement de l'ecran
#define DELAY 400
pthread_mutex_t screenlock;


// les flags 
int FLAG_LOADED;

// les maps mmoire et exploration
char *map, *memory; 
int pos_inits[MAX_INIT]; // les positions d'entres possibles

// variables lies aux dplacements
int current_position, last_position;
int direction;

// les diffrentes fentres de l'application
typedef struct {
  FL_FORM   * loader;
  void      * vdata;
  char      * cdata;
  long      ldata;

  FL_OBJECT * Texture_pixmap;
} FD_LOADER;

typedef struct {
  FL_FORM   * main;
  void      * vdata;
  char      * cdata;
  long      ldata;

  FL_OBJECT * Game_Menu;
  FL_OBJECT * Options_Menu;
  FL_OBJECT * About_Menu;
  FL_OBJECT * Clock;
  FL_OBJECT * Start_button;
  FL_OBJECT * Reset_button;
  FL_OBJECT * Quit_button;
  FL_OBJECT * Draw_map;
  FL_OBJECT * Draw_memory;
  FL_OBJECT * Carre[400];    // tableaux d'objet map
  FL_OBJECT * Mem[400];      // tableaux d'objet memory
} FD_MAIN;

FD_MAIN *fd_Main;

typedef struct {
  FL_FORM   * param;
  void      * vdata;
  char      * cdata;
  long      ldata;
} FD_PARAM;

typedef struct {
  FL_FORM   * browser;
  void      * vdata;
  char      * cdata;
  long      ldata;

  FL_OBJECT * Input_map,
    * Load_button,
    * Browse_button,
    * Exit_button;
} FD_BROWSER;

typedef struct {
  FL_FORM   * quit;
  void      * vdata;
  char      * cdata;
  long      ldata;
  
  FL_OBJECT * Question_text,
    * Yes_button,
    * No_button;
} FD_QUIT;

typedef struct {
  FL_FORM   * about;
  void      * vdata;
  char      * cdata;
  long      ldata;

  FL_OBJECT * Label_show,
    * Bot_pixmap,
    * Ok_button;
} FD_ABOUT;

// prototypes des fonctions de utils.c
FL_OBJECT * pixmap_button(int, int, int, int, int, char *, const char *);
FL_OBJECT * menu_button(int, int, int, int, int, char *);

int preemptive_handler(FL_OBJECT *, int, FL_Coord, FL_Coord, int, void *);
//static void do_tips(int, void *);
int post_handler(FL_OBJECT *, int, FL_Coord, FL_Coord, int, void *);
void set_tip(FL_OBJECT *, char *);

// prototypes des fonctions de graphic.c
extern FD_LOADER * create_form_loader(void);
extern FD_MAIN * create_form_main(void);
extern FD_PARAM * create_form_param(void);
extern FD_BROWSER * create_form_browser(void);
extern FD_QUIT * create_form_quit(void);
extern FD_ABOUT * create_form_about(void);

// prototypes des fonctions de callback.c
extern void Timeout_callback(int, void *);

void Game_Menu_callback(FL_OBJECT *, long);
void Options_Menu_callback(FL_OBJECT *, long);
void About_Menu_callback(FL_OBJECT *, long);

void Start_callback(FL_OBJECT *, long);
void Reset_callback(FL_OBJECT *, long);
void Quit_callback(FL_OBJECT *, long);
void Quit_Yes_callback(FL_OBJECT *, long);
void Quit_No_callback(FL_OBJECT *, long);

void Load_Browser_callback(FL_OBJECT *, long);
void Browse_Browser_callback(FL_OBJECT *, long);
void Exit_Browser_callback(FL_OBJECT *, long);

void About_Ok_callback(FL_OBJECT *, long);

void move_top_cb(FL_OBJECT *, long);
void move_right_cb(FL_OBJECT *, long);
void move_left_cb(FL_OBJECT *, long);
void move_bottom_cb(FL_OBJECT *, long);

// prototypes des fonctions de functions.c
void Load_Map(const char *);
void Draw_Map();
void Explorer();
void Draw_new_position(void);
void Draw_memory_position(int);

// prototypes des fonctions de sensors.c
int Sensor_Left();
int Sensor_Right();
int Sensor_Front();
void Turn_Toward(int);
void Move_Forward();
int Scan_Sensors();
void Explorer_Algo();

// prototypes des fcontions de network.c
void *network(void *);
