#include "graphic.h"

/* 
 * Fonctions pour placer les boutons et menus 
 */

FL_OBJECT * 
pixmap_button(int coord_x, int coord_y, int weight, int height, 
	      int boxtype, char *label, const char *pixmap)
{
  FL_OBJECT * obj;
  
  obj = fl_add_pixmapbutton(FL_NORMAL_BUTTON, coord_x, coord_y,
			    weight, height, label);
  
  fl_set_pixmapbutton_file(obj, pixmap);      // integre pixmap en fond
  fl_set_object_boxtype(obj, boxtype);        // type de bouton
  fl_set_object_color(obj, FL_BLACK,FL_BLUE); // couleurs contours
  fl_set_object_lalign(obj, FL_ALIGN_INSIDE); // aligne label et pixmap
  fl_set_object_lcol(obj, FL_WHITE);          // couleur fond pour label
  fl_set_object_lsize(obj, FL_NORMAL_SIZE);   // redimensionne le label
  fl_set_object_lstyle(obj, FL_BOLD_STYLE);   // label en gras
  
  return obj;
}

FL_OBJECT *
menu_button(int coord_x, int coord_y, int weight, int height,
	    int boxtype, char *label)
{
  FL_OBJECT * obj;
  
  obj = fl_add_menu(boxtype, coord_x, coord_y, weight, height, label);

  fl_set_object_lcol(obj, FL_BLACK);          // couleur fond pour label
  fl_set_object_lsize(obj, FL_NORMAL_SIZE);   // redimensionne le label
  fl_set_object_lstyle(obj, FL_BOLD_STYLE);   // label en gras
  
  return obj;
}

int
preemptive_handler(FL_OBJECT *ob, int event,
		   FL_Coord mx, FL_Coord my, int key, void *xev)
{
  switch(event)
    {
    case FL_ENTER:
      fl_set_cursor(FL_ObjWin(ob), XC_hand2);
      break;
    case FL_LEAVE:
      fl_set_cursor(FL_ObjWin(ob),FL_DEFAULT_CURSOR);
      break;
    case FL_PUSH:
    case FL_MOUSE:
    case FL_RELEASE:
    }
  
  return 0;
}

/* 
 * Fonctions pour afficher les tips 
 */

#define INTERVAL  800  /* attendre cette dure avant d'afficher le tip */
static int timeoutID;  /* ob->u_ldata pour la dure de maintien */

static void
do_tips(int id, void *p)
{
   FL_OBJECT *ob = p;

   fl_set_oneliner_color(FL_BLACK, FL_YELLOW);         // couleur du tip (label et fond)  
   fl_set_oneliner_font(FL_BOLD_STYLE, FL_SMALL_SIZE); // style police (font et taille) 

   fl_show_oneliner(ob->u_vdata, ob->form->x + ob->x,
                   ob->form->y + ob->y + ob->h + 1);
   timeoutID = fl_add_timeout(INTERVAL, do_tips, ob);
}

int
post_handler(FL_OBJECT *ob, int event,
            FL_Coord mx, FL_Coord my, int key, void *xev)
{
     if(!ob->u_vdata)
       return 0;

     if(event == FL_ENTER)
	 timeoutID = fl_add_timeout(INTERVAL,do_tips, ob);
     else if(event == FL_LEAVE || event == FL_PUSH)
       {
         fl_hide_oneliner();
         if(timeoutID)
         {
            fl_remove_timeout(timeoutID);
            timeoutID = 0;
         }
     }

     return 0;
}

void 
set_tip(FL_OBJECT *ob, char *s)
{
    ob->u_vdata = s;
    fl_set_object_posthandler(ob, post_handler);
}
